/*
 *  ngtk/c/ngtk_input_thread.c
 *
 *  Copyright (C) 1998, 1999 Gabriel Fort <gabx@tfz.net>
 *
 */

/*----------------------------------------------------------------------------

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the 
    Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
    Boston, MA  02111-1307  USA.

----------------------------------------------------------------------------*/
#include <ngtk/ngtk.h>

#include <pthread.h>
#include <semaphore.h>

#define MAX_QUEUE	32

sem_t sem;
int sem_p;
pthread_t get_loop_th;


static int input_queue[MAX_QUEUE+1];

void clear_queue(void)
{
	memset(input_queue, 0, MAX_QUEUE);
}

void push_queue(int entry)
{
	int i;

	for (i=1;i<=MAX_QUEUE;i++)
		input_queue[i] = input_queue[i-1];

	input_queue[0] = entry;
	sem_getvalue(&sem, &sem_p);
}

int pop_queue(void)
{
	sem_getvalue(&sem, &sem_p);
	return(input_queue[sem_p]);
}

int ngtk_get_keysym(void)
{
	if (use_ngtk_main == 0)
		return(ngtk_tty_getc());
	else {
		sem_wait(&sem);
		return(pop_queue());
	}
}

void * func_loop(void *arg)
{
	int i;
	int buf;

    while (1) {
        buf = ngtk_get_keysym();

        for (i=0 ; i <= key_table ; i++)
            if (buf == functable[i].keysym)
                (*functable[i].func)();
    }
}

void * get_loop(void *arg)
{
	int buf;

	while (1) {
		buf = ngtk_tty_getc();

		if (sem_p < MAX_QUEUE) {
			push_queue(buf);
			sem_post(&sem);
		}
	}
}

void ngtk_main(void)
{
	use_ngtk_main = 1;

	sem_init(&sem, 0, 0);
	clear_queue();

	pthread_create(&get_loop_th, NULL, get_loop, "get_loop");

	func_loop("func loop");
}

